@@ -206,7 +206,7 @@
   # priors
   # note the priors should be based on scaled data.
   #########################################################################################
-  cpar=state$cpar; if(is.null(cpar)) cpar=1;
+  alpha=state$alpha; if(is.null(alpha)) alpha=1;
   nburn <- mcmc$nburn;
   nsave <- mcmc$nsave;
   nskip <- mcmc$nskip;
@@ -250,7 +250,7 @@
   model.name <- "Super Survival Model:";
   foo <- .Call("PHPOAFT_BP", nburn_=nburn, nsave_=nsave, nskip_=nskip, ndisplay_=ndisplay, 
                ltr_=truncation_time, subjecti_=subjecti, t1_=t1, t2_=t2, type_=delta, X_=X.scaled, theta_=theta, 
-               beta_=beta, weight_=weight, cpar_=cpar, a0_=a0, b0_=b0, theta0_=theta0, 
+               beta_=beta, weight_=weight, cpar_=alpha, a0_=a0, b0_=b0, theta0_=theta0, 
                V0inv_=V0inv, Vhat_=Vhat, beta0_=beta0, S0inv_=S0inv, Shat_=Shat,
                l0_=min(5000,nsave/2), adapter_=2.38^2, dist_=distcode, PACKAGE = "spBayesSurv");
   
@@ -297,7 +297,7 @@
                  beta_o.scaled = beta_o.scaled,
                  beta_q.scaled = beta_q.scaled,
                  theta.scaled = theta.scaled,
-                 cpar = foo$cpar,
+                 alpha = foo$cpar,
                  maxL = maxL,
                  weight = foo$weight,
                  cpo = foo$cpo,
@@ -485,8 +485,8 @@
   if(object$prior$a0<=0){
     ans$prec <- NULL
   }else{
-    mat <- object$cpar
-    coef.p <- mean(mat)    
+    mat <- object$alpha
+    coef.p <- mean(mat); names(coef.p)="alpha";        
     coef.m <- median(mat)    
     coef.sd <- sd(mat)
     limm <- as.vector(quantile(mat, probs=c((1-CI.level)/2, 1-(1-CI.level)/2)))
@@ -513,7 +513,7 @@
   ans$ratebeta_q = object$ratebeta_q;
   ans$rateYs = object$rateYs;
   ans$ratec = object$ratec;
-  ans$cpar = object$cpar;
+  ans$alpha = object$alpha;
   ans$BF <- object$BF;
   
   class(ans) <- "summary.SuperSurvRegBayes"
@@ -541,7 +541,7 @@
                   quote = FALSE)
   }
   
-  if(x$cpar[1]==Inf){
+  if(x$alpha[1]==Inf){
     cat("\nPosterior inference of baseline parameters\n")
     message("Note: the baseline estimates are based on centered covariates")
     cat("(Adaptive M-H acceptance rate: ", x$ratetheta, "):\n", sep="")
