@@ -173,7 +173,7 @@
     v <- rep(0, length(blocki)-1);
   }
   phi = state$phi; if(is.null(phi)) phi=1;
-  phib0_prior = 1;
+  phia0_prior = 2;
   nu = prior$nu; if(is.null(nu)) nu=1;
   if(!is.null(frail.prior)){
     if(frail.prior=="grf"){
@@ -181,13 +181,14 @@
       #phi_min = (-log(0.001))^(1/nu)/maxdis; phib0_prior = -log(.95)/phi_min; phi = 1/phib0_prior;
       #cc = sqrt((-log(0.001))^(1/nu)/maxdis); phi = maxdis*cc;
       phi = (-log(0.001))^(1/nu)/maxdis; 
-      phib0_prior = 1;
+      phia0_prior = 2;
       if(!is.null(state$phi)){
         phi = state$phi;
       }
       if (phi<=0) stop("phi in state arguement should be greater than 0.")
     }
   }
+  phi0=phi;
   if(is.null(state$frail)) {
     v <- rep(0, length(blocki)-1);
   } else {
@@ -220,8 +221,8 @@
   gprior <- prior$gprior; if(is.null(gprior)) gprior <- 2*n*solve(t(Xtf1)%*%Xtf1);
   taua0 = prior$taua0; if(is.null(taua0)) taua0=1;
   taub0 = prior$taub0; if(is.null(taub0)) taub0=1;
-  phib0 = prior$phib0; if(is.null(phib0)) phib0=phib0_prior;
-  phia0 = prior$phia0; if(is.null(phia0)) phia0=phib0*phi+1;
+  phia0 = prior$phia0; if(is.null(phia0)) phia0=phia0_prior;
+  phib0 = prior$phib0; if(is.null(phib0)) phib0=(phia0-1)/phi0;
   mcmc = list(nburn=nburn, nsave=nsave, nskip=nskip, ndisplay=ndisplay)
   if(!is.null(frail.prior)){
     prior = list(maxL=maxL, a0=a0, b0=b0, siga0=siga0, sigb0=sigb0, m0=m0, S0=S0);
